// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information
use crate::{
    instruction::{PRegNum, UnitNum, CONST_ZERO_UNIT_NUM},
    unit::UnitKind,
};
use fayalite::prelude::*;

#[derive(Clone, Eq, PartialEq, Hash, Debug)]
#[non_exhaustive]
pub struct CpuConfig {
    pub unit_kinds: Vec<UnitKind>,
    pub out_reg_num_width: usize,
}

impl CpuConfig {
    pub fn unit_num_width(&self) -> usize {
        UInt::range((CONST_ZERO_UNIT_NUM + 1)..self.unit_kinds.len()).width()
    }
    pub fn unit_num(&self) -> UnitNum<DynSize> {
        UnitNum[self.unit_num_width()]
    }
    pub fn p_reg_num(&self) -> PRegNum<DynSize, DynSize> {
        PRegNum[self.unit_num_width()][self.out_reg_num_width]
    }
}
